@@ -14,11 +14,6 @@ import javax.servlet.http.HttpServletResponse;
 @SuppressWarnings("serial")
 public class LinkServlet extends MetricServlet {
 
-  @Override
-  public void init() throws ServletException {
-    super.init();
-  }
-
   @Override
   public void doGet(HttpServletRequest request,
       HttpServletResponse response) throws IOException, ServletException {
