@@ -13,11 +13,6 @@ public class StatsServlet extends AnyServlet {
 
   private static final long serialVersionUID = 6850919895248020945L;
 
-  @Override
-  public void init() throws ServletException {
-    super.init();
-  }
-
   @Override
   public void doGet(HttpServletRequest request,
       HttpServletResponse response) throws IOException, ServletException {
